/**
 * Metronome
 * http://www.kerrywong.com
 */

#define __AVR_ATmega328P__
//#define _DEBUG

#include <binary.h>
#include <HardwareSerial.h>
#include <pins_arduino.h>
#include <WConstants.h>
#include <wiring.h>
#include <wiring_private.h>
#include <WProgram.h>
#include <EEPROM/EEPROM.h>

//74HC138 LED control pins
const int PIN_LED_CTR_4 = 8; //chip pin 14
const int PIN_LED_CTR_3 = 7; //chip pin 13
const int PIN_LED_CTR_2 = 6; //chip pin 12
const int PIN_LED_CTR_1 = 5; //chip pin 11

const int PIN_LED_TEMPO_RANGE_1 = 0; //chip pin 2
const int PIN_LED_TEMPO_RANGE_2 = 1; //chip pin 3
const int PIN_LED_TEMPO_IND = 11; //chip pin 17

//speaker output pin
const int PIN_SPK = 2; //chip pin 4

const int PIN_BTN_TEMPO_DEC = 9; //chip pin 15
const int PIN_BTN_TEMPO_INC = 10; //chip pin 16

//The display range
const byte TEMPO_DISPLAY_RANGE = 16;

const int A_FREQ = 440;
const int TEMPO_FREQ = 2000;
const int BEAT_ON_DURATION = 10;

const int TEMPO[] = {
    0, //this entry indicates the option for tone A
    52, 54, 56, 58, 60, 63, 66, 69, 72, 76, 80, 84, 88, 92, 96,
    100, 104, 108, 112, 116, 120, 126, 132, 138, 144, 152, 160, 168, 176, 184, 192
};

int TempoIndex = 0;
long TempoDelay = 0;

/**
 * This serves the same purpose as delay(), but does not affect ISRs.
 */
void delayMilliseconds(int ms) {
    for (int i = 0; i < ms; i++) {
        delayMicroseconds(1000);
    }
}

/**
 * To make the beat interval more accurate,
 * the interval is adjusted by the acutal on-duration.
 */
int delayInterval(int index) {
    if (index > 0) {
        float interval = 60000.0 / (float) TEMPO[index];
        return (int) interval - BEAT_ON_DURATION;
    } else {
        return 0;
    }
}

/**
 * Initializes the digital pins
 */
void initPins() {
    pinMode(PIN_LED_CTR_1, OUTPUT);
    pinMode(PIN_LED_CTR_2, OUTPUT);
    pinMode(PIN_LED_CTR_3, OUTPUT);
    pinMode(PIN_LED_CTR_4, OUTPUT);

    pinMode(PIN_LED_TEMPO_RANGE_1, OUTPUT);
    pinMode(PIN_LED_TEMPO_RANGE_2, OUTPUT);

    pinMode(PIN_BTN_TEMPO_INC, INPUT);
    pinMode(PIN_BTN_TEMPO_DEC, INPUT);
}

/**
 * Light the LED at idx location (out of the 16 LEDs).
 */
void lightLED(int idx) {
    digitalWrite(PIN_LED_CTR_4, (idx >> 3) & 0x1);
    digitalWrite(PIN_LED_CTR_3, (idx >> 2) & 0x1);
    digitalWrite(PIN_LED_CTR_2, (idx >> 1) & 0x1);
    digitalWrite(PIN_LED_CTR_1, idx & 0x1);
}

/**
 * When the tempo index is 0-15, range indicator 1 is lit.
 * When the tempo index is 16-31, range indicator 2 is lit.
 */
void setRangeIndicator(int index) {
    int hi = (int) (index / TEMPO_DISPLAY_RANGE);

    if (hi == 0) {
        digitalWrite(PIN_LED_TEMPO_RANGE_1, HIGH);
        digitalWrite(PIN_LED_TEMPO_RANGE_2, LOW);
    } else {
        digitalWrite(PIN_LED_TEMPO_RANGE_1, LOW);
        digitalWrite(PIN_LED_TEMPO_RANGE_2, HIGH);
    }
}

void RangeIndicatorOff() {
    digitalWrite(PIN_LED_TEMPO_RANGE_1, LOW);
    digitalWrite(PIN_LED_TEMPO_RANGE_2, LOW);
}

/**
 * power on self test
*/
void POST() {
#ifdef _DEBUG
    Serial.begin(9600);
#endif
    digitalWrite(PIN_LED_TEMPO_RANGE_1, HIGH);
    digitalWrite(PIN_LED_TEMPO_RANGE_2, LOW);
}

/***************************
 * Required Arduino Routines
 ***************************/
void setup() {
    initPins();
    POST();
}

void loop() {
    if (digitalRead(PIN_BTN_TEMPO_DEC) == HIGH) {
        delayMilliseconds(10); //debouncing the button
        if (digitalRead(PIN_BTN_TEMPO_DEC) == HIGH) {
            if (TempoIndex > 0)
                TempoIndex--;
            lightLED(TempoIndex);
            TempoDelay = delayInterval(TempoIndex);
            setRangeIndicator(TempoIndex);
            delayMilliseconds(200);
        }
    } else if (digitalRead(PIN_BTN_TEMPO_INC) == HIGH) {
        delayMilliseconds(10); //debouncing the button
        if (digitalRead(PIN_BTN_TEMPO_INC) == HIGH) {
            if (TempoIndex < TEMPO_DISPLAY_RANGE * 2 - 1)
                TempoIndex++;
            lightLED(TempoIndex);
            TempoDelay = delayInterval(TempoIndex);
            setRangeIndicator(TempoIndex);
            delayMilliseconds(200);
        }
    } else {
        if (TempoIndex == 0) { //"A"
            tone(PIN_SPK, A_FREQ);
        } else {
            noTone(PIN_SPK);
            setRangeIndicator(TempoIndex);
            tone(PIN_SPK, TEMPO_FREQ);
            delayMilliseconds(BEAT_ON_DURATION);
            RangeIndicatorOff();
            noTone(PIN_SPK);
            delayMilliseconds(TempoDelay);
        }
    }
}
